/***************************************************************************
 *   Copyright (C) 2003-2005 by Diego "Flameeyes" Pettenò                  *
 *   flameeyes@users.berlios.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "joyconf.h"
#include "kvba.h"

#include <SDL.h>

#include <iostream>

#include <qpushbutton.h>
#include <qtextstream.h>

#include <kfiledialog.h>
#include <kconfig.h>

// defines taken from SDL_Test sourcecode
#define GETHATCODE(event) (((event.jhat.which+1)<<12)|(event.jhat.hat<<2)|\
  (event.jhat.value & SDL_HAT_UP ? 0 : event.jhat.value & SDL_HAT_DOWN ? 1 : event.jhat.value & SDL_HAT_RIGHT ? 2 : event.jhat.value & SDL_HAT_LEFT ? 3 : 0))

#define GETBUTTONCODE(event) (((event.jbutton.which+1)<<12)|(event.jbutton.button+0x80))

#define GETAXISCODE(event) (((event.jaxis.which+1)<<12)|(event.jaxis.axis<<1)|(event.jaxis.value > 16384 ? 1 : event.jaxis.value < -16384 ? 0 : 0))

JoyConf::JoyConf(QWidget *parent, const char *name, WFlags f)
 : uiJoyConf(parent, name, f)
{
}


JoyConf::~JoyConf()
{
}

void JoyConf::buttonClicked()
{
	QPushButton *s = (QPushButton*)sender();

	SDL_Event event;
	SDL_WaitEvent(&event);
   	switch(event.type)
	{
		case SDL_JOYHATMOTION:
			s->setText(QString::number(GETHATCODE(event)));
			return;
		case SDL_JOYBUTTONDOWN:
			s->setText(QString::number(GETBUTTONCODE(event)));
			return;
		case SDL_JOYAXISMOTION:
			s->setText(QString::number(GETAXISCODE(event)));
			return;
		case SDL_KEYUP:
			if(event.key.keysym.sym == SDLK_ESCAPE)
				s->setText("");
			else
				s->setText(QString::number(event.key.keysym.sym));
			return;
	}

	do SDL_Delay(100);
	while(SDL_PollEvent(&event));
}

void JoyConf::reject()
{
	QDialog::reject();
}

void JoyConf::accept()
{
	QString cfgfile = KFileDialog::getSaveFileName(
    	QString::null,
        QString::null,
        this,
        tr("Select where to save")
        );

	QFile f(cfgfile);
	f.open(IO_WriteOnly);
	QTextStream str(&f);

	KGlobal::config()->setGroup("/general");
	QString sep = KGlobal::config()->readNumEntry( "vbaversion", 0) >= 1 ? "=" : " ";

	str	<< "Joy0_Left" << sep << buttonLeft->text() << endl
		<< "Joy0_Right" << sep << buttonRight->text() << endl
		<< "Joy0_Up" << sep << buttonUp->text() << endl
		<< "Joy0_Down" << sep << buttonDown->text() << endl
		<< "Joy0_A" << sep << buttonA->text() << endl
		<< "Joy0_B" << sep << buttonB->text() << endl
		<< "Joy0_L" << sep << buttonLTrig->text() << endl
		<< "Joy0_R" << sep << buttonRTrig->text() << endl
		<< "Joy0_Start" << sep << buttonStart->text() << endl
		<< "Joy0_Select" << sep << buttonSelect->text() << endl
		<< "Joy0_Speed" << sep << buttonSpeed->text() << endl
		<< "Joy0_Capture" << sep << buttonCapture->text() << endl;

	buttonUp->setText("");
	buttonDown->setText("");
	buttonLeft->setText("");
	buttonRight->setText("");
	buttonLTrig->setText("");
	buttonRTrig->setText("");
	buttonA->setText("");
	buttonB->setText("");
	buttonStart->setText("");
	buttonSelect->setText("");
	buttonSpeed->setText("");
	buttonCapture->setText("");
	QDialog::accept();
}
